<!DOCTYPE html>
<html>

<title>Situação da Gripe</title>
<link rel="stylesheet" href="/static/libs/bootstrap/bootstrap.min.css">
<link rel="stylesheet" href="/static/css/custom.css">
<link rel="icon"
      type="image/png"
      href="/static/img/info_gripe_icon.ico"
/>

<!-- README.HTML STYLE HERE :END-->

</head>

<body>

<div class="navbar navbar-dark bg-primary" role="navigation">
  <iframe src="/super-header" class="super-header" scrolling="no"></iframe>

  <div class="container-fluid">
    <div class="navbar-header">
      <a class="navbar-brand" href="./"><strong>Monitoramento de casos
        reportados de síndrome respiratória aguda
        grave (SRAG) no SINAN</strong></a>
      <a href="/help"
         style="display:inline;font-weight:bold;color:white;cursor:pointer;"
      >[Ajuda]</a>
    </div>
  </div>
</div>

<div class="document container-fluid">
  <div class="row my-3">
    <div class="col-md-3"></div>
    <div class="col-md-6">
      <div class="card p-3">
        <h1 class="message-box-error">
          <!-- CONTENT HERE:START -->
          Página não encontrada.
          <!-- CONTENT HERE:END -->
        </h1>
      </div>
    </div>
    <div class="col-md-3"></div>
  </div>
</div>
</body>
</html>