<!DOCTYPE html>
<html>
<head>
  <title>Situação da Gripe</title>
  <base target="_parent">
  <link rel="icon"
      type="image/png"
      href="/static/img/info_gripe_icon.ico"
  />
<!--
  <link rel="stylesheet"
        href="http://www.ensp.fiocruz.br/portal-ensp/_estilos/ensp_basico.css"
        type="text/css" media="all"/>
-->

  <link rel="stylesheet"
        href="http://www.ensp.fiocruz.br/portal-ensp/_estilos/ensp_barra-fiocruz.css"
        type="text/css" media="all"/>

  <style>
    html, document, body {
      margin: 0;
      padding: 0;
      background: #ffffff!important;
    }
  </style>
</head>

<style type="text/css">

  img.align-left, .figure.align-left, object.align-left {
    float: left ;
    margin-right: 1em }

  img.align-right, .figure.align-right, object.align-right {
    float: right ;
    margin-left: 1em }

</style>


<body>
  <div id="barra-brasil" style="background:#7F7F7F; height: 20px; padding:0 0 0 10px;display:block;">
    <ul id="menu-barra-temp" style="list-style:none;">
        <li style="display:inline; float:left;padding-right:10px; margin-right:10px; border-right:1px solid #EDEDED"><a href="http://brasil.gov.br" style="font-family:sans,sans-serif; text-decoration:none; color:white;">Portal do Governo Brasileiro</a></li>
        <li><a style="font-family:sans,sans-serif; text-decoration:none; color:white;" href="http://epwg.governoeletronico.gov.br/barra/atualize.html">Atualize sua Barra de Governo</a></li>
    </ul>
  </div>

  <div id="prologo">

     <ul class="barra-fiocruz">
        <li><a class="logo-fiocruz" href="http://www.fiocruz.br/"><span>Fiocruz</span></a></li>
        <li><a class="fale-com" href="http://portal.fiocruz.br/pt-br/content/fale-conosco"><span>Fale com a Fiocruz</span></a></li>
        <li><a class="logo-ensp" href="http://www.fiocruz.br/"><span>Fiocruz</span></a></li>
    </ul>

  </div>

  <div style="margin-top: 10px; margin-bottom: 10px;">
      <a href="/">
          <img src="/static/img/info_gripe.png" alt="InfoGripe" height="60px"/>
      </a>
      <a href="http://www.procc.fiocruz.br/">
        <img class="align-right" src="/static/img/procc.png" alt="PROCC" height="60px" />
      </a>
      <a href="http://emap.fgv.br/">
        <img class="align-right" src="/static/img/fgv-emap.png" alt="FGV-EMap" height="60px"/>
      </a>
  </div>

  <script src="./static/libs/jquery/jquery.js"></script>
  <script defer="defer" src="//barra.brasil.gov.br/barra.js" type="text/javascript"></script>
</body>
</html>
