.table-hover tbody tr:hover td, .table-hover tbody tr:hover th {
    background-color: #f5f5f5;
}
.table th, .table td {
    text-align: left;
    vertical-align: top;
    border-top: 1px solid #ddd;
}
.table td {
    padding: 2px 2px 2px 20px!important;
    line-height: 12px!important;
    font-size: 12px!important;
}
user agent stylesheettd, th {
    display: table-cell;
    vertical-align: inherit;
}