html, body {
  overflow-x: hidden;
}

html, document, body {
    margin: 0!important;
    padding: 0!important;
    background: #dfdfdf;
}

iframe.super-header {
  padding: 0;
  margin: 0;
  border: 0;
  width: 100%;
  height: 140px;
}

.navbar {
  margin: 0!important;
  padding: 0!important;
}

.navbar-fixed {
    border-width: 0 0 1px;
    -webkit-transform: translate3d(0,0,0);
    -o-transform: translate3d(0,0,0);
    transform: translate3d(0,0,0);
    border-radius: 0;
}

@media (min-width: 768px)
.navbar-fixed {
    border-radius: 0;
}

.hidden {
    display: none!important;
}

input[type=search] {
    border: 1px solid #dfdfdf!important;
}

input.tab{
    margin: 0!important;
    font-size: 12px!important;
    padding: 5px;
    color: #666666;
    font-size: 12px!important;
}


input.tab.selected {
    font-weight: bold;
    color: #333333;
}

.criteria-box {
  display: inline-block;
  padding-right: 20px;
  vertical-align: top;
}

.content-height {
  height: 450px;
}

.align-left {
  float: left;
}

.align-right {
  float: right;
}

.message-box-error {
    vertical-align: middle!important;
    text-align: center!important;
}

.print-card {
    height: 150mm;
    width: 210mm;
}
