.legend-container #legend {
  background-color: rgba(255, 255, 255, 1.0);
  -webkit-border-radius: 5px;
  -moz-border-radius: 5px;
  -ms-border-radius: 5px;
  -o-border-radius: 5px;
  border-radius: 5px;
}

.legend-container .legend-toggle {
  cursor: pointer;
  width: 26px;
  height: 26px;
  -webkit-border-radius: 4px;
  border-radius: 4px;
  background-color: rgba(255, 255, 255, 1.0);
}
.legend-container .legend-toggle-icon {
  /*optionally use an icon image for the toggle button*/
  /*background: url('images/layers.png') no-repeat center center;*/
  font-family: font-awesome;
  font-size: 16px!important;
  text-align: center!important;
  padding-left: 10px!important;
}
.legend-collapsed #legend {
  display: none;
}
.legend-collapsed .legend-toggle {
  display: block;
}

#legend {
  padding: 10px;
  opacity: .8;
  filter: alpha(opacity=80); /* For IE8 and earlier */
}

.legend-square {
  width: 10px;
  height: 10px;
}
