<!DOCTYPE html>
<html>

<title>Situação da Gripe</title>
<link rel="stylesheet" href="./static/libs/bootstrap/bootstrap.min.css"/>
<link rel="stylesheet" href="./static/css/custom.css"/>
<link rel="stylesheet" href="./static/css/help.css"/>
<link rel="icon"
      type="image/png"
      href="./static/img/info_gripe_icon.ico"
/>
<!-- README.HTML STYLE HERE :END-->

</head>

<body>

<div class="navbar navbar-dark bg-primary" role="navigation">
  <iframe src="./super-header" class="super-header" scrolling="no"></iframe>

  <div class="container-fluid">
    <div class="navbar-header">
      <a class="navbar-brand" href="./"><strong>Monitoramento de casos
        reportados de síndrome respiratória aguda
        grave (SRAG) no SINAN</strong></a>
      <a href="/help"
         style="display:inline;font-weight:bold;color:white;cursor:pointer;"
      >[Ajuda]</a>
    </div>
  </div>
</div>

<div class="document container-fluid">
  <div class="row">
    <div class="col-md-12 my-3">

      <div class="card">
        <h3 class="card-header">InfoGripe</h3>
        <div class="card-block p-3">
          <!-- README.HTML HERE :BEGIN -->

          <div class="section" id="infogripe">
            <p>InfoGripe é uma iniciativa para monitorar e apresentar níveis de
              alerta para os casos reportados de Síndrome Respiratória Aguda
              Grave (SRAG) no SINAN, o Sistema de Informação de Agravos de
              Notificação (www.saude.gov.br/sinan).
              Os dados são apresentados por estado e por regiões de vigilância
              para síndromes gripais.</p>
            <p>Este é um produto da parceria entre pesquisadores do Programa de
              Computação Científica da Fundação Oswaldo Cruz (Fiocruz, PROCC),
              da Escola de Matemática Aplicada da Fundação Getúlio Vargas (FGV,
              EMAp), no Rio de Janeiro, e do GT-Influenza da Secretaria de
              Vigilância Sanitária do Ministério da Saúde (GT-Influenza, SVS,
              MS).</p>
            <ul class="simple">
              <li>Software livre: GNU General Public License v3</li>
              <li>Documentação:
                <a class="reference external"
                   href="https://fludashboard.readthedocs.io">https://fludashboard.readthedocs.io</a>.
              </li>
            </ul>
            <div class="section" id="funcionalidades">
              <h5>Funcionalidades</h5>
              <ul class="simple">
                <li>Nowcast de incidência semanal;</li>
                <li>Limiares de atividade;</li>
                <li>Distribuição etária dos casos notificados;</li>
                <li>Nível de atividade por temporada;</li>
                <li>Séries históricas.</li>
              </ul>
            </div>
            <div class="section" id="tutoriais">
              <h5>Tutoriais</h5>
              <ul>
                <li><p class="first">Panorama semanal:</p>
                  <p><a class="reference external"
                        href="https://youtu.be/OWbvpxsJnpQ">https://youtu.be/OWbvpxsJnpQ</a>
                  </p>
                </li>
              </ul>
              <iframe width="854" height="480"
                      src="https://www.youtube.com/embed/OWbvpxsJnpQ"
                      frameborder="0" allow="autoplay; encrypted-media"
                      allowfullscreen></iframe>
            </div>
            <br/>
          </div>
          <div class="section" id="descricao-das-funcionalidades">
            <h4>Descrição das funcionalidades</h4>
            <p>InfoGripe apresenta níveis de atividade e informação sobre a
              série temporal de incidência por semana epidemiológica (SE) e por
              temporada.
              Estas informações podem ser visualizadas pelo panorama <strong>Detalhado
                (semana)</strong> e <strong>Resumido (ano)</strong>. Cada
              panorama é composto por quatro painéis:</p>
            <ul class="simple">
              <li>Mapa do país;</li>
              <li>Curva de incidência (por 100mil habitantes);</li>
              <li>Tabela de incidência (por 100mil habitantes);</li>
              <li>Distribuição etária (por 100mil habitantes);</li>
            </ul>
            <p>Em cada panorama, as informações podem ser visualizadas por
              estado (UF) ou região.</p>
            <div class="section" id="panorama-detalhado-semana">
              <h5>Panorama Detalhado (semana):</h5>
              <ul class="simple">
                <li>Mapa do país (superior esquerdo)</li>
              </ul>
              <blockquote>
                <p>Cada estado/região possui uma cor de acordo com o nível de
                  atividade para a SE selecionada:</p>
                <ul class="simple">
                  <li>Atividade baixa (verde): incidência abaixo do limiar
                    pré-epidêmico;
                  </li>
                  <li>Atividade epidêmica (amarelo): incidência acima do limiar
                    pré-epidêmico e abaixo do limiar de atividade alta;
                  </li>
                  <li>Atividade alta (laranja): incidência acima do limiar de
                    atividade alta e abaixo do de atividade muito alta;
                  </li>
                  <li>Atividade muito alta (vermelho): incidência acima do
                    limiar de atividade muito alta.
                  </li>
                </ul>
              </blockquote>
              <ul class="simple">
                <li>Curva de incidência (superior direito)</li>
              </ul>
              <blockquote>
                <p>Este painel apresenta a série temporal de incidência
                  reportada (curva preta sólida) para a temporada
                  correspondente, com uma linha vertical indicando a SE
                  selecionada. Estimativas de incidência, quando disponíveis,
                  são apresentadas com curva sólida em vermelho, com o
                  intervalo de confiança de 95% representado por linhas
                  vermelhas pontilhadas. A probabilidade de cada nível de
                  atividade também é apresentado em formato de texto no canto
                  superior esquerdo do gráfico. As cores do mapa correspondem
                  ao nível de maior probabilidade. Além da incidência, neste
                  gráfico apresentamos também os limiares de atividade
                  correspondente:</p>
                <ul class="simple">
                  <li>Limiar pré-epidêmico (linha azul tracejada): nível de
                    atividade que indica, quando ultrapassado, o início da
                    transmissão sustentada para a temporada atual. Após superar
                    este limiar, a incidência tende a apresentar crescimento
                    gradual até atingir o pico de incidência para aquele ano;
                  </li>
                  <li>Limiar de atividade alta (linha verde tracejada): nível
                    de atividade a partir do qual a incidência é considerada
                    alta para aquela localidade (estado/região). Calculado com
                    base em estimativa para o percentil 90 da distribuição de
                    incidência semanal histórica;
                  </li>
                  <li>Nível de atividade muito alta (linha vermelha tracejada):
                    nível de atividade a partir da qual a incidência é
                    considerada muito elevada para aquela localidade. Calculado
                    com base na estimativa para o percentil 97.5 da
                    distribuição de incidência semanal histórica;
                  </li>
                </ul>
              </blockquote>
              <ul class="simple">
                <li>Esquema de cores do pano de fundo da curva de incidência
                </li>
              </ul>
              <blockquote>
                <p>O pano de fundo demarca os níveis de atividade típicos para
                  cada semana. Isto é, o perfil histórica da incidência para
                  cada semana. Isto permite indetificar o padrão sazonal típico
                  , facilitando a identificação do período usual de atividade
                  epidêmica.</p>
                <ul class="simple">
                  <li>Atividade semanal baixa (área verde): incidência abaixo
                    do percentil 10 para cada SE;
                  </li>
                  <li>Atividade semanal baixa à moderada (área amarela):
                    incidência entre os percentís 10 e 50 (mediana) para cada
                    SE;
                  </li>
                  <li>Atividade semanal moderada à alta (área laranja):
                    incidência entre os percentís 50 e 90 para cada SE;
                  </li>
                  <li>Atividade semanal alta (área vermelha): incidência acima
                    do percentil 90 para cada SE.
                  </li>
                </ul>
                <p>Quando a incidência em uma determinada semana está dentro da
                  região de atividade alta (fundo vermelho), indica que para
                  aquela SE a atividade é atípicamente alta, independentemente
                  dos limiares de atividade. Esta informação é útil para
                  detectar temporadas em que o período epidêmico se inicia
                  antes do usual, por exemplo. Vê a temporada de 2016 no estado
                  do Rio Grande do Sul para um exemplo deste comportamento.</p>
              </blockquote>
              <ul class="simple">
                <li>Tabela de incidência (inferior esquerdo)</li>
              </ul>
              <blockquote>
                <p>Incidência para a localidade correspondente na SE
                  selecionada, juntamente com o intervalo de confiança de 90%
                  quando valor for estimado. Juntamenteda incidência e nome da
                  localidade, esta tabela apresenta a situação dos dados
                  selecionados:</p>
                <ul class="simple">
                  <li>Estável: dados reportados considerados suficientemente
                    próximos do total casos notificados. Estes dados estão
                    sujeitos a pequenas alterações futuras;
                  </li>
                  <li>Estimado: dados reportados baseado em estimativa para a
                    oportunidade de digitação. Isto é, baseado no número de
                    notificações já digitadas no sistema (dados incompletos) e
                    o tempo típico entre notificação na unidade de saúde e
                    digitação desta notificação no sistema. Estes dados podem
                    sofrer alterações futuras, tornando-se estáveis após
                    algumas semanas;
                  </li>
                  <li>Incompleto: dados reportados ainda não são estáveis em
                    função do padrão de oportunidade de digitação observado na
                    localidade selecionada e nosso sistema não é capaz de
                    produzir estimativas confiáveis. Estes dados podem sofrer
                    grandes alterações futuras, tornando-se estáveis após
                    algumas semanas.
                  </li>
                </ul>
              </blockquote>
              <ul class="simple">
                <li>Distribuição etária e por gênero (inferior direito)</li>
              </ul>
              <blockquote>
                <p>Gráfico de barras para a incidência reportada (sem uso de
                  estimativa) por faixa etária e gênero, para a SE
                  selecionada.</p>
                <ul class="simple">
                  <li>Mulheres: barra azul;</li>
                  <li>Homens: barra laranja;</li>
                  <li>Total: barra verde</li>
                </ul>
                <p>Estas distribuições estão sujeitas a alterações conforme
                  situação reportada na tabela de incidência. Neste painel não
                  apresentamos estimativas por faixa etária, reportando apenas
                  as notificações já digitadas, sejam os dados estáveis ou
                  incompletos.</p>
              </blockquote>
            </div>
            <div class="section" id="panorama-resumido-ano">
              <h5>Panorama Resumido (ano):</h5>
              <p>Este panorama apresenta a situação anual com base nos níveis
                de atividade descritos no panorama detalhado.</p>
              <ul class="simple">
                <li>Mapa do país (superior esquerdo)</li>
              </ul>
              <blockquote>
                <p>Cada localidade apresenta cor de acordo com a atividade
                  regsitrada na temporada selecionada seguindo os seguintes
                  critérios:</p>
                <ul class="simple">
                  <li>Atividade baixa (verde): incidência abaixo do limiar
                    pré-epidêmico durante toda a temporada;
                  </li>
                  <li>Atividade epidêmica (amarelo): incidência superou o
                    limiar pré-epidêmico ao menos em uma SE porém não superou o
                    limiar de atividade alta;
                  </li>
                  <li>Atividade alta (laranja): incidência superou o limiar de
                    atividade alta ou muito alta entre 1 a 4 SEs;
                  </li>
                  <li>Atividade muito alta (vermelho): incidência superou o
                    limiar de atividade alta ou muito alta em ao menos 5 SEs.
                  </li>
                </ul>
              </blockquote>
              <ul class="simple">
                <li>Curva de incidência (superior direito)</li>
              </ul>
              <blockquote>
                <p>Este painel apresenta a série temporal de incidência
                  reportada (curva preta sólida) para a temporada
                  correspondente. Estimativas de incidência, quando
                  disponíveis, são apresentadas com curva sólida em vermelho,
                  com o intervalo de confiança de 95% representado por linhas
                  vermelhas pontilhadas. A probabilidade de cada nível de
                  atividade também é apresentado em formato de texto no canto
                  superior esquerdo do gráfico. As cores do mapa correspondem
                  ao nível de maior probabilidade. Além da incidência, neste
                  gráfico apresentamos também os limiares de atividade
                  correspondente:</p>
                <ul class="simple">
                  <li>Limiar pré-epidêmico (linha azul tracejada): nível de
                    atividade que indica, quando ultrapassado, o início da
                    transmissão sustentada para a temporada atual. Após superar
                    este limiar, a incidência tende a apresentar crescimento
                    gradual até atingir o pico de incidência para aquele ano;
                  </li>
                  <li>Limiar de atividade alta (linha verde tracejada): nível
                    de atividade a partir do qual a incidência é considerada
                    alta para aquela localidade (estado/região). Calculado com
                    base em estimativa para o percentil 90 da distribuição de
                    incidência semanal histórica;
                  </li>
                  <li>Nível de atividade muito alta (linha vermelha tracejada):
                    nível de atividade a partir da qual a incidência é
                    considerada muito elevada para aquela localidade. Calculado
                    com base na estimativa para o percentil 97.5 da
                    distribuição de incidência semanal histórica;
                  </li>
                </ul>
              </blockquote>
              <ul class="simple">
                <li>Esquema de cores do pano de fundo da curva de incidência
                </li>
              </ul>
              <blockquote>
                <p>O pano de fundo demarca os níveis de atividade típicos para
                  cada semana. Isto é, o perfil histórica da incidência para
                  cada semana. Isto permite indetificar o padrão sazonal típico
                  , facilitando a identificação do período usual de atividade
                  epidêmica.</p>
                <ul class="simple">
                  <li>Atividade semanal baixa (área verde): incidência abaixo
                    do percentil 10 para cada SE;
                  </li>
                  <li>Atividade semanal baixa à moderada (área amarela):
                    incidência entre os percentís 10 e 50 (mediana) para cada
                    SE;
                  </li>
                  <li>Atividade semanal moderada à alta (área laranja):
                    incidência entre os percentís 50 e 90 para cada SE;
                  </li>
                  <li>Atividade semanal alta (área vermelha): incidência acima
                    do percentil 90 para cada SE.
                  </li>
                </ul>
                <p>Quando a incidência em uma determinada semana está dentro da
                  região de atividade alta (fundo vermelho), indica que para
                  aquela SE a atividade é atípicamente alta, independentemente
                  dos limiares de atividade. Esta informação é útil para
                  detectar temporadas em que o período epidêmico se inicia
                  antes do usual, por exemplo. Vê a temporada de 2016 no estado
                  do Rio Grande do Sul para um exemplo deste comportamento.</p>
              </blockquote>
              <ul class="simple">
                <li>Tabela de incidência (inferior esquerdo)</li>
              </ul>
              <blockquote>
                <p>Incidência para a localidade correspondente na temporada
                  selecionada, juntamente com o intervalo de confiança de 90%
                  quando valor for estimado. Juntamenteda incidência e nome da
                  localidade, esta tabela apresenta a situação dos dados
                  selecionados:</p>
                <ul class="simple">
                  <li>Estável: dados reportados considerados suficientemente
                    próximos do total casos notificados. Estes dados estão
                    sujeitos a pequenas alterações futuras;
                  </li>
                  <li>Incompleto: dados reportados ainda não são estáveis em
                    função do padrão de oportunidade de digitação observado na
                    localidade selecionada e nosso sistema não é capaz de
                    produzir estimativas confiáveis. Estes dados podem sofrer
                    grandes alterações futuras, tornando-se estáveis após
                    algumas semanas.
                  </li>
                </ul>
              </blockquote>
              <ul>
                <li><p class="first">Distribuição etária e por gênero (inferior
                  direito)</p>
                  <p>Gráfico de barras para incidência reportada (sem uso de
                    estimativa) por faixa etária e gênero, para a temporada
                    selecionada.</p>
                </li>
              </ul>
              <blockquote>
                <ul class="simple">
                  <li>Mulheres: barra azul;</li>
                  <li>Homens: barra laranja;</li>
                  <li>Total: barra verde</li>
                </ul>
                <p>Estas distribuições estão sujeitas a alterações conforme
                  situação reportada na tabela de incidência. Neste painel não
                  apresentamos estimativas por faixa etária, reportando apenas
                  as notificações já digitadas, sejam os dados estáveis ou
                  incompletos.</p>
              </blockquote>
            </div>
          </div>

          <!-- README.HTML HERE:END -->
        </div>
      </div>

    </div>
  </div>
</div>
</body>
</html>
